Rc4

Struct Rc4 

pub struct Rc4<const KEY_LEN: usize, D: DropStrategy = Zeroize>(/* private fields */);
An algorithm that performs RC4 encryption and decryption. This algorithm is generic over drop strategy.

RC4 is a stream cipher that uses a variable-length key (1-256 bytes). The key is stored alongside the encrypted data and is used to reproduce the keystream for decryption at runtime.