pub struct ByteArray;Expand description
Mode marker type indicating the encrypted data should be treated as a byte array.
When used as the M type parameter of Encrypted<A, M, N>, dereferencing
returns &[u8; N] (a reference to the raw byte array).